# -*- coding: utf-8 -*-
import os
from django.conf import settings

from rest_framework import permissions
from rest_framework import views
from rest_framework.response import Response
from rest_framework.settings import api_settings

from . import serializers


class GenericListPermission(permissions.DjangoModelPermissions):
    perms_map = {
        'GET': ['filesystem.list_filesystem'],
        'OPTIONS': [],
    }

    def has_permission(self, request, view):
        return True


class FilesystemList(views.APIView):
    permission_classes = (GenericListPermission,)
    mypath = settings.FILESYSTEM_ROOT_DIR
    serializer_class = serializers.FilesystemListSerializer
    pagination_class = api_settings.DEFAULT_PAGINATION_CLASS

    def get_queryset(self):
        return []

    def get_serializer(self, *args, **kwargs):
        serializer_class = self.serializer_class
        return serializer_class(*args, **kwargs)

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

    def list(self, request, *args, **kwargs):
        elements = self.get_elements(request, *args, **kwargs)

        page = self.paginate_queryset(elements)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(elements, many=True)
        return Response(serializer.data)

    @property
    def paginator(self):
        """
        The paginator instance associated with the view, or `None`.
        """
        if not hasattr(self, '_paginator'):
            if self.pagination_class is None:
                self._paginator = None
            else:
                self._paginator = self.pagination_class()
        return self._paginator

    def paginate_queryset(self, queryset):
        """
        Return a single page of results, or `None` if pagination is disabled.
        """
        if self.paginator is None:
            return None
        return self.paginator.paginate_queryset(queryset, self.request, view=self)

    def get_paginated_response(self, data):
        """
        Return a paginated style `Response` object for the given output data.
        """
        assert self.paginator is not None
        return self.paginator.get_paginated_response(data)

    def get_elements(self, request, *args, **kwargs):
        code = kwargs.get('code', '')
        path = kwargs.get('path', '')
        mypath = os.path.join(self.mypath, code, path)
        list_dirs_and_files = []
        for dir_file in os.listdir(mypath):
            if os.path.isfile(os.path.join(mypath, dir_file)):
                list_dirs_and_files.append({
                    'name': dir_file,
                    'type': 'f',
                    'link': '',
                })
            else:
                list_dirs_and_files.append({
                    'name': dir_file,
                    'type': 'd',
                    'link': os.path.join('http://127.0.0.1:8000/api/filesystem/filesystem/', code, path, dir_file)
                })
        return list_dirs_and_files
